unit iniparser;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

Type
  RModelGun = record
     DA_archetype: string;
     material_library: string;
     envmap_material: string;
     LODranges: string;
     HP_child: string;
  end;
  RModelMunition = record
     DA_archetype: string;
     material_library: string;
     envmap_material: string;
     LODranges: string;
     HP_trail_parent: string;
  end;

  var
    LModelGun: array of RModelGun;
    LModelMunition: array of RModelMunition;

    function ParseGameIni(const Path: string): Boolean;
    function ParseEditorIni(const Path: string): Boolean;
    function ParseWeaponIni(const Path: string; const GunName: string): Boolean;

implementation

uses
  filestructure, BIniReader, Unit1, unitweaponlist, uniterror, strutils, Graphics;

Type
  TStringArray = array of string;

function ExtendedTrim(const S:string):string;inline;
begin
  result:=trim(StringReplace(s,#160,#32,[rfReplaceAll]));
end;

procedure AddToIniFileList(Var StrArray: TStringArray; const S: string);inline;
begin
  SetLength(StrArray,Length(StrArray)+1);
  StrArray[High(StrArray)]:=S;
end;

function ParseGameIni(const Path: string): Boolean;
var
  IniFile: TStrings;
  i: Cardinal;
  j: integer;
  CurrentSection, ColonPos, BracketLPos, BracketRPos, EqualPos: Word;
  IniFileListPos: Word=0;
  ParsedString: string;
  Sections: array [0..9] of string = ('data','debris','effect','lightanim','sound','lootcrate','weapontype','shieldequipconsts','explosion','gun');
  IniFileTypes: array [0..5] of string = ('constants','sounds','debris','equipment','effects','weaponmoddb'); //lightanim hardcoded
  IniFileList: TStringArray;
  SectionFound: Boolean=False;
  AddedSection: Boolean=False;
begin
  AddToIniFileList(IniFileList,Path+'\exe\freelancer.ini');
  AddToIniFileList(IniFileList,Path+'\data\fx\lightanim.ini');

  result:=True;

  repeat
  begin
    if (FileExists(IniFileList[IniFileListPos])) then
    begin
      IniFile:=BIniReader.LoadBiniFile(IniFileList[IniFileListPos]);
      for i:=0 to IniFile.Count-1 do
      begin
        ParsedString:=ExtendedTrim(IniFile.Strings[i]);
        ColonPos:=AnsiPos(';', ParsedString); // Kommentar
        if (ColonPos=1) then
         Continue
        else
        if (ColonPos>1) then
          ParsedString:=Copy(ParsedString,0,ColonPos-1);
        BracketLPos:=AnsiPos('[', ParsedString);  // Sektion
        BracketRPos:=AnsiPos(']', ParsedString);
        if ((BracketLPos <> 0) and (BracketRPos <> 0)) then
        begin
          ParsedString:=LowerCase(Copy(ParsedString,BracketLPos+1,BracketRPos-(BracketLPos+1)));
          for CurrentSection:=0 to High(Sections) do
            if (ParsedString=Sections[CurrentSection]) then
            begin
              SectionFound:=True;
              AddedSection:=False;
              break;
            end
            else
              SectionFound:=False;
        end
        else
          if SectionFound then // Inhalt
          begin
            EqualPos:=AnsiPos('=', ParsedString);
            with Form1 do
              case CurrentSection of

                0: // [data] - Freelancer.ini
                  for j:=0 to High(IniFileTypes) do
                    if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))=IniFileTypes[j]) then
                      AddToIniFileList(IniFileList,Path+'\data\'+Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                1: // [debris]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') then
                    ins_gun_debris_type.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                2: // [effect]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') then
                  begin
                    ins_gun_flash_name.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                    if AnsiContainsText(Copy(ParsedString,EqualPos+1,MaxInt), 'proj') or
                       AnsiContainsText(Copy(ParsedString,EqualPos+1,MaxInt), 'drive') then
                      ins_projeffect.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                    if AnsiContainsText(Copy(ParsedString,EqualPos+1,MaxInt), 'impact') or
                       AnsiContainsText(Copy(ParsedString,EqualPos+1,MaxInt), 'exp') then
                      ins_hiteffect.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                  end;
                    // Weitere Zuweisungen via Assign in btn_selectFL

                3: // [lightanim]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='name') then
                    ins_gun_lightanim.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                4: // [sound]
                  begin
                    if not AddedSection then
                    begin
                      SetLength(Sounds,Length(Sounds)+1);
                      AddedSection:=True;
                    end;
                    case LowerCase(Trim(Copy(ParsedString,1,EqualPos-1))) of
                      'nickname':
                        begin
                          ins_mun_sound.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                          Sounds[High(Sounds)].name:=(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                          if AnsiContainsText(Sounds[High(Sounds)].name, 'fire') then
                          begin
                            ins_sound.Items.Add(Sounds[High(Sounds)].name);
                            SetLength(NormalShotSndIndex,Length(NormalShotSndIndex)+1);
                            NormalShotSndIndex[High(NormalShotSndIndex)]:=High(Sounds);
                          end;
                        end;
                      'file': Sounds[High(Sounds)].path:=select_freelancer.FileName+'\data\'+Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                    end;
                  end;
                    // Weitere Zuweisungen via Assign in btn_selectFL

                5: // [lootcrate]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') then
                    ins_mun_lootmodel.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                6: // [weapontype]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') then
                    ins_mun_weaptype.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                7: // [shieldequipconsts]
                  if (UpperCase(Trim(Copy(ParsedString,1,EqualPos-1)))='HULL_DAMAGE_FACTOR') then
                    energydmgfactor:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                8: // [explosion]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') then
                    ins_gun_separtexpl.Items.Add(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));

                9: // [gun]
                  if (LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') then
                  begin
                    ParsedString:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                    SetLength(WeaponList,Length(WeaponList)+1);
                    WeaponList[High(WeaponList)].name:=ParsedString;
                    WeaponList[High(WeaponList)].path:=IniFileList[IniFileListPos];
                    Form3.list_weaponlist.Items.Add(ParsedString);
                  end;
              end;
          end;
      end;
      IniFile.Free;
    end
    else
    begin
      Form2.syn_error.Lines.Append('ERROR - Could not find '+IniFileList[IniFileListPos]+'!');
      Result:=False;
    end;
    inc(IniFileListPos);
  end;
  until IniFileListPos>High(IniFileList);

  SetLength(IniFileList, 0);
end;

function ParseEditorIni(const Path: string): Boolean;
var
  IniFile: TStrings;
  i: Cardinal;
  CurrentSection, ColonPos, BracketLPos, BracketRPos, EqualPos: Word;
  ParsedString: string;
  Sections: array [0..1] of string = ('gun','munition');
  SectionFound: Boolean=False;
  Bitmap: TJPEGImage;
begin
  if FileExists(Path) then
  begin
    Bitmap:=TJPEGImage.Create;
    Bitmap.Width:=Form1.img_archetype.Width;
    Bitmap.Height:=Form1.img_archetype.Height;

    IniFile:=TStringList.Create;
    IniFile.LoadFromFile(Path);
    for i:=0 to IniFile.Count-1 do
    begin
      ParsedString:=ExtendedTrim(IniFile.Strings[i]);
      ColonPos:=AnsiPos(';', ParsedString); // Kommentar
      if (ColonPos=1) then
       Continue
      else
      if (ColonPos>1) then
        ParsedString:=Copy(ParsedString,0,ColonPos-1);
      BracketLPos:=AnsiPos('[', ParsedString);  // Sektion
      BracketRPos:=AnsiPos(']', ParsedString);
      if ((BracketLPos <> 0) and (BracketRPos <> 0)) then
      begin
        ParsedString:=LowerCase(Copy(ParsedString,BracketLPos+1,BracketRPos-(BracketLPos+1)));
        for CurrentSection:=0 to High(Sections) do
          if (ParsedString=Sections[CurrentSection]) then
          begin
            SectionFound:=True;
            case ParsedString of
              'gun': AddGunModels;
              'munition': AddMunitionModels;
            end;
            break;
          end
          else
            SectionFound:=False;
      end
      else
        if SectionFound then // Inhalt
        begin
          EqualPos:=AnsiPos('=', ParsedString);
          with Form1 do
            case CurrentSection of

              0: // [gun]
                case LowerCase(Trim(Copy(ParsedString,1,EqualPos-1))) of
                  'model_path':
                    begin
                      GunModels[High(GunModels)].model_path:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      ParsedString:=ExtractFileName(GunModels[High(GunModels)].model_path);
                      Form1.ins_gun_archetype.Items.Add(ParsedString);
                      GunModels[High(GunModels)].imageIndex:=AddIconToList('icons\'+ParsedString+'.jpg')
                    end;
                  'material_path': GunModels[High(GunModels)].material_path:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'envmap_path': GunModels[High(GunModels)].envmap_path:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'lodranges': GunModels[High(GunModels)].LODRanges:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'hp_child': GunModels[High(GunModels)].HP_child:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                end;

              1: // [munition]
                case LowerCase(Trim(Copy(ParsedString,1,EqualPos-1))) of
                  'model_path':
                    begin
                      MunitionModels[High(MunitionModels)].model_path:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      ParsedString:=ExtractFileName(MunitionModels[High(MunitionModels)].model_path);
                      Form1.ins_mun_archetype.Items.Add(ParsedString);
                      MunitionModels[High(MunitionModels)].imageIndex:=AddIconToList('icons\'+ParsedString+'.jpg');
                    end;
                  'material_path': MunitionModels[High(MunitionModels)].material_path:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'lodranges': MunitionModels[High(MunitionModels)].LODRanges:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'hp_trail_parent': MunitionModels[High(MunitionModels)].HP_trail_parent:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                end;
            end;
        end;
    end;
    IniFile.Free;
    Result:=True;
    Bitmap.Free;
  end
  else
  begin
    Form2.syn_error.Lines.Append('ERROR - Could not find Weapon Creator'+#39+'s '+Path+'!');
    Result:=False;
  end;

end;

procedure SetEndLine(const Line: Cardinal);inline;
begin
  if Line>EndLine then
    EndLine:=Line;
end;
procedure SetStartLine(const Line: Cardinal);inline;
begin
  if Line<StartLine then
    StartLine:=Line;
end;

function ParseWeaponIni(const Path: string; const GunName: string): Boolean;
var
  IniFile: TStrings;
  i: Cardinal=0;
  SectionLine: Cardinal=0;
  CurrentSection, ColonPos, BracketLPos, BracketRPos, EqualPos: Word;
  ParsedString: string;
  Sections: array [0..3] of string = ('gun','munition','explosion','motor');
  SectionFound: Boolean=False;
  FoundGun: byte=0;     // 0 - nicht gefunden
  FoundMunition: byte=0;// 1 - gefunden
  FoundMotor: byte=0;   // 2 - abgeschlossen
  FoundExplosion: byte=0;
  FoundGood: byte=0;
begin
  if FileExists(Path) then
  begin
    IniFile:=BIniReader.LoadBiniFile(Path);
    StartLine:=IniFile.Count-1;
    EndLine:=0;

    while i<IniFile.Count do
    begin
      ParsedString:=ExtendedTrim(IniFile.Strings[i]);
      ColonPos:=AnsiPos(';', ParsedString); // Kommentar
      if (ColonPos=1) then
      begin
        Inc(i);
        Continue
      end
      else
      if (ColonPos>1) then
        ParsedString:=Copy(ParsedString,0,ColonPos-1);
      BracketLPos:=AnsiPos('[', ParsedString);  // Sektion
      BracketRPos:=AnsiPos(']', ParsedString);
      if ((BracketLPos <> 0) and (BracketRPos <> 0)) then
      begin
        ParsedString:=LowerCase(Copy(ParsedString,BracketLPos+1,BracketRPos-(BracketLPos+1)));
        if FoundGun=1 then
        begin
          FoundGun:=2;
          if i<>0 then
            SetEndLine(i-1);
        end;

        if FoundMunition=1 then
        begin
          FoundMunition:=2;
          if i<>0 then
            SetEndLine(i-1);
        end;

        if FoundMotor=1 then
        begin
          FoundMotor:=2;
          if i<>0 then
            SetEndLine(i-1);
        end;

        if FoundExplosion=1 then
        begin
          FoundExplosion:=2;
          if i<>0 then
            SetEndLine(i-1);
        end;

        if FoundGood=1 then
        begin
          FoundGood:=2;
          if i<>0 then
            SetEndLine(i-1);
        end;

        for CurrentSection:=0 to High(Sections) do
          if (ParsedString=Sections[CurrentSection]) then
          begin
            SectionFound:=True;
            SectionLine:=i;
            break;
          end
          else
            SectionFound:=False;
      end
      else
        if SectionFound then // Inhalt
        begin
          EqualPos:=AnsiPos('=', ParsedString);
          with Form1 do
            case CurrentSection of

              0: // [gun]
                if FoundGun=1 then
                  with Gun do
                    case LowerCase(Trim(Copy(ParsedString,1,EqualPos-1))) of
                      'ids_name': idname:=StrToQWord(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'ids_info': idinfo:=StrToQWord(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'nickname': nickname:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'LODranges': LODranges:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'DA_archetype': DA_archetype:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'material_library': material_library:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'envmap_material': envmap_material:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'hp_gun_type':
                        case LowerCase(Trim(Copy(ParsedString,EqualPos+1,MaxInt))) of
                          'hp_gun': hp_gun_type:=1;
                          'hp_gun_special_1': hp_gun_type:=2;
                          'hp_gun_special_2': hp_gun_type:=3;
                          'hp_gun_special_3': hp_gun_type:=4;
                          'hp_gun_special_4': hp_gun_type:=5;
                          'hp_gun_special_5': hp_gun_type:=6;
                          'hp_gun_special_6': hp_gun_type:=7;
                          'hp_gun_special_7': hp_gun_type:=8;
                          'hp_gun_special_8': hp_gun_type:=9;
                          'hp_gun_special_9': hp_gun_type:=10;
                          'hp_gun_special_10': hp_gun_type:=11;
                          'hp_turret': hp_gun_type:=12;
                          'hp_turret_special_1': hp_gun_type:=13;
                          'hp_turret_special_2': hp_gun_type:=14;
                          'hp_turret_special_3': hp_gun_type:=15;
                          'hp_turret_special_4': hp_gun_type:=16;
                          'hp_turret_special_5': hp_gun_type:=17;
                          'hp_turret_special_6': hp_gun_type:=18;
                          'hp_turret_special_7': hp_gun_type:=19;
                          'hp_turret_special_8': hp_gun_type:=20;
                          'hp_turret_special_9': hp_gun_type:=21;
                          'hp_turret_special_10': hp_gun_type:=22;
                          'hp_torpedo': hp_gun_type:=23;
                          'hp_torpedo_special_1': hp_gun_type:=24;
                          'hp_torpedo_special_2': hp_gun_type:=25;
                          else hp_gun_type:=0;
                        end;
                      'HP_child': HP_child:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'hit_pts': hit_pts:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'explosion_resistance': explosion_resistance:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'explosion_arch': explosion_arch:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'debris_type': debris_type:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'parent_impulse': parent_impulse:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'child_impulse': child_impulse:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'volume': volume:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'mass': mass:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'damage_per_fire': damage_per_fire:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'power_usage': power_usage:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'refire_delay': refire_delay:=1/StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'muzzle_velocity': muzzle_velocity:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'toughness': toughness:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'flash_particle_name': flash_particle_name:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'flash_radius': flash_radius:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'light_anim': light_anim:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'projectile_archetype': projectile_archetype:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'separation_explosion': separation_explosion:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'auto_turret': auto_turret:=StrToBool(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'turn_rate': turn_rate:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'lootable': lootable:=StrToBool(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                      'dry_fire_sound': dry_fire_sound:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                      'dispersion_angle': dispersion_angle:=StrToFloat(Trim(Copy(ParsedString,EqualPos+1,MaxInt)));
                    end
                else
                  if (FoundGun=0) and ((LowerCase(Trim(Copy(ParsedString,1,EqualPos-1)))='nickname') and
                     (LowerCase(Trim(Copy(ParsedString,EqualPos+1,MaxInt)))=GunName)) then
                  begin
                    FoundGun:=1;
                    i:=SectionLine+1;
                    SetStartLine(SectionLine);
                    Continue;
                  end;

              {1: // [munition]
                case LowerCase(Trim(Copy(ParsedString,1,EqualPos-1))) of
                  'model_path':
                  'material_path': MunitionModels[High(MunitionModels)].material_path:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'lodranges': MunitionModels[High(MunitionModels)].LODRanges:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                  'hp_trail_parent': MunitionModels[High(MunitionModels)].HP_trail_parent:=Trim(Copy(ParsedString,EqualPos+1,MaxInt));
                end;  }

            end;
        end;
      Inc(i);
    end;
    if EndLine=0 then
      SetEndLine(IniFile.Count-1);
    IniFile.Free;
    Result:=True;
  end
  else
  begin
    Form2.syn_error.Lines.Append('ERROR - Could not find '+Path+'!');
    Result:=False;
  end;

end;

end.

